#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : nlp_app
# @Time         : 2020/6/8 11:27 上午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import os
from collections import OrderedDict

import numpy as np

import jieba.analyse  as ja
from gensim.models.fasttext import load_facebook_model

_norm = lambda x, ord=1: x / np.linalg.norm(x, ord, axis=len(x.shape) > 1, keepdims=True)

import time
from datetime import datetime
import pandas as pd
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse
from starlette.status import *

ROUTE = "/nlp"
app = FastAPI(
    debug=True,
    openapi_url=f"{ROUTE}/openapi.json",
    docs_url=f"{ROUTE}/docs",
    redoc_url=f"{ROUTE}/redoc",
    swagger_ui_oauth2_redirect_url=f"{ROUTE}/docs/oauth2-redirect"
)

cmd = f"wget http://cdn.cnbj1.fds.api.mi-img.com/browser-algo-nanjing/data/wv/skipgram.title -O skipgram.title"
os.popen(cmd).read()

wv = load_facebook_model("./skipgram.title").wv


def topk_words_vec(q):
    vec = np.zeros(200)
    ws = ja.tfidf(q, topK=8)

    if ws:
        for w in ws:
            vec += wv[w] / len(ws)
        vec = _norm(vec)
    return " ".join(map(str, vec.tolist()))


@app.post("/nlp/wv/fasttext")
async def report(kwargs: dict):
    dic = OrderedDict()

    id2sent = kwargs['id2word']

    data = {i: topk_words_vec(sent) for i, sent in id2sent.items()}
    dic['data'] = data

    return dic


if __name__ == '__main__':
    import socket
    import uvicorn

    me = socket.gethostname() == 'yuanjie-Mac.local'
    main_file = __file__.split('/')[-1].split('.')[0]

    # app参数必须是个字符串才能开启热更新
    uvicorn.run(app=f"{main_file}:app", host='0.0.0.0', port=8000, workers=1, debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : xindao_ann
# @Time         : 2020/11/10 5:40 下午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : mi-run - task xindao_ann --python ~/anaconda3/bin/python

import os
import time
import datetime
import requests

from milvus import DataType

from tql.pipe import *

os.environ['LOG_PATH'] = f'/home/work/yuanjie/{Path(__file__).name}.log'

from tql.utils.logger_utils import logger
from mi.db import Mongo, ANN, Hive

conn = Hive.Hive('/home/work/keytab/h_browser.keytab').conn

mongo = Mongo()
xindao_ann = mongo.db.xindao_ann


def get_vector(titles=["小米", "大米"], sleep=0.3):
    time.sleep(sleep)
    url = 'http://simbert-vector-cl33103.c3.ingress.mice.cc.d.xiaomi.net/simbert'
    json = {"texts": titles}
    r = requests.post(url, json=json)
    return r.json()['vectors']


category_map = {k: xindao_ann.find_one({'category': k})['category_'] for k in xindao_ann.distinct('category')}

ann = ANN('10.119.33.90')

dfs = []
for i in [-1, -2, -3]:
    date_1 = str(get_date(i))[:10].replace('-', '')
    sql = f"""
    select id, xindaoid, title, category[0] category
    from feeds.content_databus_xindao_batch_idx
    where date = {date_1}
    and xindaoid is not null
    and category[0] is not null 
    """
    dfs.append(pd.read_sql(sql, conn))

df_xd = pd.concat(dfs).drop_duplicates(ignore_index=True)

df_xd['xindaoid'] = df_xd['xindaoid'].astype(int)
is_in_mongo = df_xd['xindaoid'].progress_map(lambda x: True if xindao_ann.find_one({'xindaoid': x}) else False)
df_xd = df_xd[~is_in_mongo]

if len(df_xd) > 0:
    logger.info(f"新增内容{len(df_xd)}条，更新ANN服务")

    vecs = (
            df_xd['title'].tolist() | xgroup_by_step(10) |
            xThreadPoolExecutor(get_vector, 10) | xlist
    )

    df_xd['vector'] = np.row_stack(vecs).tolist()

    for category in df_xd['category']:
        category_map[category] = category_map.get(category, max(category_map.values()) + 1)

    df_xd['category_'] = df_xd['category'].map(category_map)

    # mongo insert
    xindao_ann.insert_many(df_xd.to_dict('r'), False)

    # ann instert
    collection_name = 'demo'
    ids = df_xd.xindaoid.tolist()
    hybrid_entities = [
        {"name": "category_", "values": df_xd.category_.tolist(), "type": DataType.INT32},
        {"name": "embedding", "values": df_xd.vector.tolist(), "type": DataType.FLOAT_VECTOR},
    ]

    _ = ann.client.insert(collection_name, hybrid_entities, ids)

else:
    logger.info("无新增内容，无需更新ANN服务")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-App.
# @File         : demo
# @Time         : 2019-12-10 17:24
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

"""
配合：https://github.com/notifiers/notifiers
https://www.jianshu.com/p/2945634fe349
"""
from loguru import logger

trace = logger.add('x.log')  # 输出日志到文件
logger.debug('this is a debug message')
logger.remove(trace)  # 中断写入文件
logger.debug('写不进去了')

# 时间格式化
logger.add("file.log", format="{time:YYYY-MM-DD at HH:mm:ss} | {level} | {message}")

# 字符串格式化
logger.info('If you are using Python {}, prefer {feature} of course!', 3.6, feature='f-strings')

# compression
logger.add("file_Y.log", compression="zip")  # 保存zip格式

logger.add("somefile.log", enqueue=True)  # 异步写入
logger.add("somefile.log", serialize=True)  # 序列化为json

# retention
logger.add('runtime.log', retention='10 days')  # 一段时间后会清空

# rotation
from loguru import logger

logger.add('runtime_{time}.log', rotation="500 MB")  # 每 500MB 存储一个文件
logger.add('runtime_{time}.log', rotation='00:00')  # 每天 0 点新创建一个 log 文件
logger.add('runtime_{time}.log', rotation='1 week')  # 每隔一周创建一个 log 文件


# Traceback
@logger.catch
def my_function(x, y, z):
    # An error? It's caught anyway!
    return 1 / (x + y + z)


from loguru import logger

trace = logger.add('runtime_{time}.log', rotation="100 MB", retention='10 days')
logger.debug('this is a debug message')
if __name__ == '__main__':
    with logger.catch():
        my_function()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : simbert
# @Time         : 2020-04-08 20:22
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import re
import time
from collections import OrderedDict
from datetime import datetime
import pandas as pd
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse
from starlette.status import *
from traceback import format_exc  # https://www.cnblogs.com/klchang/p/4635040.html

ROUTE = "/bert"
app = FastAPI(
    debug=True,
    openapi_url=f"{ROUTE}/openapi.json",
    docs_url=f"{ROUTE}/docs",
    redoc_url=f"{ROUTE}/redoc",
    swagger_ui_oauth2_redirect_url=f"{ROUTE}/docs/oauth2-redirect"
)

#################################################################################
import numpy as np
import keras
from bert4keras.models import build_transformer_model
from bert4keras.tokenizers import Tokenizer
from bert4keras.snippets import sequence_padding
from loguru import logger


# import os
# os.environ['TF_KERAS'] = '1'
# cli
import argparse

parser = argparse.ArgumentParser(description='BERT APP')
parser.add_argument('-d', '--bertDir', type=str, help='bert dir')
args = parser.parse_args()
BERT_DIR = args.bertDir
if BERT_DIR:
    logger.info(f"BERT_DIR: {BERT_DIR}")

config_path = f'{BERT_DIR}/bert_config.json'
checkpoint_path = f'{BERT_DIR}/bert_model.ckpt'
dict_path = f'{BERT_DIR}/vocab.txt'

# 建立分词器
tokenizer = Tokenizer(dict_path, do_lower_case=True)

# 建立加载模型
bert = build_transformer_model(
    config_path,
    checkpoint_path,
    with_pool='linear',
    application='unilm',
    return_keras_model=False  # True: bert.predict([np.array([token_ids]), np.array([segment_ids])])
)

encoder = keras.models.Model(bert.model.inputs, bert.model.outputs[0])


# seq2seq = keras.models.Model(bert.model.inputs, bert.model.outputs[1])

def normalize(x):
    return x / np.linalg.norm(x, 2, axis=len(x.shape) > 1, keepdims=True)


def sentence_encode(texts: list, batch_size=128):
    X, S = [], []
    for t in texts:
        x, s = tokenizer.encode(t)
        X.append(x)
        S.append(s)
    X = sequence_padding(X)
    S = sequence_padding(S)

    vec = encoder.predict([X, S], batch_size)
    return normalize(vec).tolist()


@app.get("/bert/simbert")
def simbert(request: Request):
    dic = OrderedDict()
    date = datetime.now()
    dic['date'] = date.__str__()[:10]
    dic['datetime'] = date.__str__()
    dic['timestamp'] = date.timestamp()

    texts = request.query_params.get("texts", [])
    if texts:
        dic['vectors'] = sentence_encode(texts)
    else:
        dic['vectors'] = [[]]

    return dic


@app.post("/bert/simbert")
def simbert(kwargs: dict):
    dic = OrderedDict()
    date = datetime.now()
    dic['date'] = date.__str__()[:10]
    dic['datetime'] = date.__str__()
    dic['timestamp'] = date.timestamp()

    texts = kwargs.get("texts", [])
    if texts:
        dic['vectors'] = sentence_encode(texts)
    else:
        dic['vectors'] = [[]]

    return dic


if __name__ == '__main__':
    import socket
    import uvicorn

    me = socket.gethostname() == 'yuanjie-Mac.local'
    main_file = __file__.split('/')[-1].split('.')[0]

    # app参数必须是个字符串才能开启热更新
    uvicorn.run(app=f"{main_file}:app", host='0.0.0.0', port=8000, workers=1, debug=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : tql-Python.
# @File         : s
# @Time         : 2019-11-01 10:23
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : https://docs.streamlit.io/api.html?highlight=markdown#display-progress-and-status
# https://docs.streamlit.io/cli.html
# streamlit run your_script.py --server.port 80
# 8051
# https://github.com/streamlit/streamlit/tree/develop/lib/streamlit/hello

import os
import time
import datetime
import numpy as np
import pandas as pd
import streamlit as st
from mi.db import Mongo
from bson import json_util

m = Mongo()
# side
date = st.sidebar.date_input('设置日期', datetime.datetime.today()).__str__()
biz = st.sidebar.selectbox('业务场景', ('browser', 'xiangkan', 'newhome'), index=1)
collection = m.db[f"push.{biz}"]
st.sidebar.markdown("> Params:")
st.sidebar.json({'date': date, 'biz': biz})

# mongo
st.sidebar.markdown('---')
st.sidebar.markdown('**Mongo Find**')
mongo_filter = st.sidebar.text_input('', {'date': date})  # **kwargs
_ = collection.find_one(eval(mongo_filter))
st.sidebar.json(json_util.dumps(_))

# TODO: 增加其他特性 email/常用api/

# body
st.title("Push实验报告")  # st.markdown("# Push Report")


# @st.cache(suppress_st_warning=True)
def get_data(jobid, date):
    try:
        _ = collection.find({'jobid': jobid, 'date': date})
        df = pd.DataFrame(list(_)[0]['data'])
    except Exception as e:
        st.markdown(f"`{e}`")
        df = pd.DataFrame()
    return df


def multi_select_df(df, subset=None, axis=0, color="yellow"):
    cols = st.multiselect('MultiSelect', df.columns.tolist(), df.columns.tolist())
    st.dataframe(df[cols].style.highlight_max(subset, color, axis))


df = get_data('想看vs大数据', date).set_index("experimentName")
multi_select_df(df)

df = df.drop('date', 1).T
multi_select_df(df, axis=1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : test
# @Time         : 2020-03-11 14:56
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import time
import yaml
from kazoo.client import KazooClient

zk = KazooClient(hosts="tjwqstaging.zk.hadoop.srv:11000")
zk.start()



def watcher(data, stat):  # (data, stat, event)
    cfg = yaml.safe_load(data)
    print(f"Config Version: {stat.version}")
    print(cfg)


def zk_watcher(path='/mipush/hive'):
    zk.DataWatch(path)(watcher)


zk_watcher()

if __name__ == '__main__':
    # zk_schedule()
    while True:
        time.sleep(3)
        print('ok')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : mi.
# @File         : iapp
# @Time         : 2020/10/22 11:01 上午
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 

import time
import uvicorn
import pandas as pd

from datetime import datetime
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse
from starlette.status import *

from collections import OrderedDict
from traceback import format_exc  # https://www.cnblogs.com/klchang/p/4635040.html


class App(object):

    def __init__(self, verbose=True):
        self.app = FastAPI()
        self.verbose = verbose

    def run(self, app=None, host="0.0.0.0", port=8000, workers=1, access_log=True, debug=True, reload=True, **kwargs):
        """

        :param app:   app字符串可开启热更新
        :param host:
        :param port:
        :param workers:
        :param access_log:
        :param debug:
        :param reload:
        :param kwargs:
        :return:
        """
        uvicorn.run(
            app if app else self.app,
            host=host, port=port, workers=workers, access_log=access_log, debug=debug, reload=reload
        )

    def add_route(self, path='/xxx', func=lambda x='demo': x, method="GET", **kwargs):

        handler = self._handler(func, method, **kwargs)

        self.app.api_route(path=path, methods=[method])(handler)

    def _handler(self, func, method='GET', result_key='result', **kwargs):
        """

        :param func:
        :param method:
            get -> request: Request
            post -> kwargs: dict
        :param result_key:
        :return:
        """
        if method == 'GET':
            async def handler(request: Request):
                input = dict(request.query_params)
                return self._try_func(input, func, result_key, **kwargs)

        elif method == 'POST':
            async def handler(kwargs_: dict):
                input = kwargs_
                return self._try_func(input, func, result_key, **kwargs)

        else:
            async def handler():
                return {'Warning': 'method not in {"GET", "POST"}'}

        return handler

    def _try_func(self, kwargs, func, result_key='result', **kwargs_):
        input = kwargs
        output = OrderedDict()

        if self.verbose:
            output['RequestParams'] = input

        try:
            output['suceess'] = 1
            output[result_key] = func(**input)

        except Exception as error:
            output['suceess'] = 0
            output['Error'] = error
            output['ErrorPlus'] = format_exc().strip()

        return output



app = App()
app.add_route('/get', lambda **kwargs: kwargs, method="GET", result_key="GetResult")
app.add_route('/post', lambda **kwargs: kwargs, method="POST", result_key="PostResult")

app = app.app

if __name__ == '__main__':
    import socket
    import uvicorn

    me = socket.gethostname() == 'yuanjie-Mac.local'
    main_file = __file__.split('/')[-1].split('.')[0]

    uvicorn.run(app=f"{main_file}:app", host='0.0.0.0', port=9000, workers=1, debug=True, reload=True)

    # app.run(port=9000, debug=False, reload=False)
    # app_.run(f"{main_file}:app", port=9000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : Python.
# @File         : base_app
# @Time         : 2020-03-21 15:40
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


import time
import uvicorn
import pandas as pd

from datetime import datetime
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse
from starlette.status import *

from collections import OrderedDict
from traceback import format_exc  # https://www.cnblogs.com/klchang/p/4635040.html

app = FastAPI()


class App(object):

    def __init__(self, verbose=True):
        self.app = FastAPI()
        self.verbose = verbose

    def run(self, app=None, host="0.0.0.0", port=8000, workers=1, access_log=True, debug=True, reload=True, **kwargs):
        """

        :param app:   app字符串可开启热更新
        :param host:
        :param port:
        :param workers:
        :param access_log:
        :param debug:
        :param reload:
        :param kwargs:
        :return:
        """
        uvicorn.run(
            app if app else self.app,
            host=host, port=port, workers=workers, access_log=access_log, debug=debug, reload=reload
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : Python.
# @File         : zk_app
# @Time         : 2020-03-11 14:39
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : 


# !/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : Python.
# @File         : app
# @Time         : 2020-03-11 13:54
# @Author       : yuanjie
# @Email        : yuanjie@xiaomi.com
# @Software     : PyCharm
# @Description  : http://zkview.d.xiaomi.net/clusters/tjwqstaging/nodes?path=%2Fmipush%2Fhive&isAdmin=false


import time
import pandas as pd
from typing import Optional
from fastapi import FastAPI, Form, Depends, File, UploadFile
from pydantic import BaseModel
from starlette.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.responses import \
    RedirectResponse, FileResponse, HTMLResponse, PlainTextResponse
from starlette.status import *

from kazoo.client import KazooClient
import yaml
from traceback import format_exc  # https://www.cnblogs.com/klchang/p/4635040.html
from collections import OrderedDict

ROUTE = "/zk"
app = FastAPI(
    debug=True,
    openapi_url=f"{ROUTE}/openapi.json",
    docs_url=f"{ROUTE}/docs",
    redoc_url=f"{ROUTE}/redoc",
    swagger_ui_oauth2_redirect_url=f"{ROUTE}/docs/oauth2-redirect"
)


@app.get("/zk/{zk_url}/{zk_path}")
async def zk_cfg(zk_url, zk_path):
    dic = OrderedDict()

    zk_path = zk_path.replace('.', '/')

    zk = KazooClient(hosts=zk_url)
    zk.start()

    try:
        data, stat = zk.get(zk_path)
        cfg = yaml.safe_load(data)

        dic[zk_path] = cfg
        dic['children'] = zk.get_children(zk_path)

    except Exception  as e:
        dic['error'] = format_exc()

    return dic


if __name__ == '__main__':
    import socket
    import uvicorn

    me = socket.gethostname() == 'yuanjie-Mac.local'
    main_file = __file__.split('/')[-1].split('.')[0]

    # app参数必须是个字符串才能开启热更新
    uvicorn.run(app=f"{main_file}:app", host='0.0.0.0', port=8000, workers=1, reload=True, debug=True)
